
clear all
close all


%% load param
startup;
p = load_param();


%% load data

[ X Yo ] = load_data(p.data_dir);
[ Xo ] = transform_data_pca( p.data_dir, X, p.PCA_NRG);
clear X;

Xall_orig = cat(1,Xo{:});
Yall_orig = cat(1,Yo{:});
Xtest = Xo;
Ytest = Yo;

N_datasets = size(Yo,2);
RDIM = size(Xall_orig,2);


%% create folders for results

if ~exist(p.res_dir,'dir'), mkdir(p.res_dir); end
if ~exist(p.res_dir_theta,'dir'), mkdir(p.res_dir_theta); end

%-------------------------------------------------------
  
switch p.METHOD
 
  case 'svm-gen'
        
        %% leave-one-out
        for itest = 1:N_datasets

            fprintf('\nTraining %d/%d ... ',itest,N_datasets);
            %----- test set ------
            Xn=Xtest{itest};
            Yn=Ytest{itest};
        
            %--- training set ---
            [ Xall Yall ] = load_all_data_leave_one_out(Xo, Yo, itest ); 
            fprintf(' %d x %d ...', size(Xall, 1), size(Xall,2));
        
            %--- select training subdata ( 50% pos e 50% neg)
            if p.PROP_DATA < 1.0
                [Xall Yall] = select_sub_data(Xall, Yall, p.PROP_DATA, balance_data_sel); 
            else
                fprintf('\nDEBUG : selecting %.1f of datapoints : %d \n', 100.0*p.PROP_DATA, length(Yall));
            end
            %---------------------------------------------------------

            %--- set weight ---
            if abs(length(find(Yall==1)) - length(find(Yall==-1))) <= 1
                weight_p = 1;
                weight_n = 1;
            else
                weight_p = 1.0/length(find(Yall==1))
                weight_n = 1.0/length(find(Yall==-1))
            end


            paramstr = sprintf('-t 0 -c %.6f -w1 %.6f -w-1 %.6f', p.C_class, weight_p, weight_n);
            model = svmtrain(Yall, Xall, paramstr);
            w_all = model.SVs' * model.sv_coef;
            b_all = -model.rho;
            z = w_all'*Xn' + b_all;
            
            %--- eval ---
            Z{itest} = z;
            Yeval{itest} = Yn';
            z = sign(z);
            acc(itest) = eval_accuracy(Yn, z');    

            %--- save theta
	     file_w = fullfile(p.res_dir_theta,sprintf('w_%d.mat',itest));
            file_b = fullfile(p.res_dir_theta,sprintf('b_%d.mat',itest));
            save(file_w,'w_all');
            save(file_b,'b_all');

    
        end

	%% -------------------------------------------------------------------------------------

    case 'svm-reg'
        
        if ~exist(p.res_dir_kernel,'dir')
            mkdir(p.res_dir_kernel)
        end
        
        %% 1/3) train/test individual classifier on each source dataset
         
        for i=1:N_datasets
            
            Xi = Xo{i};
            Yi = Yo{i};

            %--- set weights    
            if abs(length(find(Yi==1)) - length(find(Yi==-1))) <= 1
                weight_p = 1.0;
                weight_n = 1.0;
            else
                weight_p = 1.0/length(find(Yall_orig==1));
                weight_n = 1.0/length(find(Yall_orig==-1));
            end
            paramstr = sprintf('-t 0 -c %.6f -w1 %.6f -w-1 %.6f', p.C_class, weight_p, weight_n);
            disp(['training source ' num2str(i) '/' num2str(N_datasets) ' with [' paramstr '] ... ']);
                
            model = svmtrain(Yi, Xi, paramstr); 
            wi = model.SVs' * model.sv_coef;
            bi = -model.rho;
            svi = full(model.SVs);
            w_orig{i} = wi;
            b_orig{i} = bi;
            SV{i} = svi;
            
            z = wi'*Xtest{i}' + bi;
            z = sign(z);
            acc_i(i) = eval_accuracy(Ytest{i}, z');           

            %--- save
	    file_wi = fullfile(p.res_dir_theta,sprintf('wi_%d.mat',i));
            file_bi = fullfile(p.res_dir_theta,sprintf('bi_%d.mat',i));
            file_svi = fullfile(p.res_dir_theta,sprintf('SVi_%d.mat',i));                            
            save(file_wi,'wi');
            save(file_bi,'bi');
            save(file_svi,'svi');
            
        end
        

        %% 2/3) build Kernel matrix
        
        [Kernel_all_train Kernel_all_test ] = compute_kernel(p.KERNEL_FUN,Xo,SV,p.SVMREG_GAMMA,p.C_class,p.res_dir_kernel);
        %------------------------------------------------------------------------------------------------
        
        %% leave-one-out TPT
      
 	for itest = 1:N_datasets

            fprintf('\nLoading data for regression %d/%d ... ',itest,N_datasets);
            
            %----- test set ------
            Xn=Xtest{itest};
            Yn=Ytest{itest};
            wn = w_orig{itest};
            bn = b_orig{itest};
            Ktmp = Kernel_all_test(itest,:);
            Kernel_test = [Ktmp(1,1:itest-1) Ktmp(1,itest+1:end)];
            
            %--- training set ---
            clear X;
            clear Y;
            idx = [1:itest-1 itest+1:N_datasets];
            for i = 1:1:N_datasets-1
                j=idx(i);
                X{i} = Xo{j};
                Y{i} = Yo{j};
                w{i} = w_orig{j};
                b{i} = b_orig{j};
            end            
            Ktmp = [Kernel_all_train(:,1:itest-1) Kernel_all_train(:,itest+1:end)];
            Kernel_train = [Ktmp(1:itest-1,:); Ktmp(itest+1:end,:)];
            %---------------------------------------------------------

            %% learn a regression model ---
            fprintf('\nRegression %d/%d ... ',itest,N_datasets);
 
            K =  [ (1:N_datasets-1)' , Kernel_train ];
            KK = [ 1 , Kernel_test  ];

            [w_out, b_out] = POR(w, b, K, KK, p.C_reg, p.SVMREG_P);
            
            fprintf('\nEval accuracy %d/%d ... ',itest,N_datasets);
          
            z = w_out*Xn' + b_out;
            Z{itest} = z ;
            Yeval{itest} = Yn';
            z=sign(z);
            acc(itest) = eval_accuracy(Yn, z');
            
            %--- save
            file_w_out = fullfile(p.res_dir_theta,sprintf('w_out_%d.mat',itest));
            file_b_out = fullfile(p.res_dir_theta,sprintf('b_out_%d.mat',itest));
            save(file_w_out,'w_out');
            save(file_b_out,'b_out');
            
        end

%%--------------------------------------------------------------------------------------------------------
%%--------------------------------------------------------------------------------------------------------

end


Yall = cat(2,Yeval{:})';
Zall = cat(2,Z{:})';
final_acc = eval_accuracy(Yall, Zall);

disp(' ');
disp('-----------------------------------------------');

disp(['Method : ' p.METHOD ])
switch p.METHOD
 
    case 'svm-gen'
        disp(['   C_class : ' num2str(p.C_class) ])
        

    case 'svm-reg'
        disp(['   C_class : ' num2str(p.C_class) ])
        disp(['   Kernel : ' p.KERNEL_FUN ])
        disp(['   gamma : ' num2str(p.SVMREG_GAMMA) ])
        disp(['   regr P : ' num2str(p.SVMREG_P) ])
        disp(['   regr C : ' num2str(p.C_reg) ])
end

disp(' ')
disp(['   final accuracy = ' num2str(final_acc) ]);
  
disp('-----------------------------------------------');
